/*
 * Copyright 2016 Stefan Christ <contact@stefanchrist.eu>
 * License: no license, public domain!
 *
 * Complie with
 *     $ gcc wait.c -std=gnu99 -o wait
 *
 * Usage:
 *    shell 1 $ ./wait
 *    shell 2 $ strace -p $(pidof wait)
 *    # Again in shell 1, type 'hi<enter>'
 *    # Then open shell 3
 *    shell 3 $ kill -s SIGUSR1 $(pidof wait)
 *
 *    # Output of program:
 *    read value: [0] 0x68
 *    read value: [1] 0x69
 *    read value: [2] 0xa
 *    Handling signal 10
 *    error in poll: 4 (Interrupted system call)
 *
 *    # Complete output of strace:
 *    read(0, "hi\n", 10)                     = 3
 *    fstat64(1, {st_mode=S_IFCHR|0620, st_rdev=makedev(136, 13), ...}) = 0
 *    mmap2(NULL, 4096, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, -1, 0) = 0xb777d000
 *    write(1, "read value: [0] 0x68\n", 21)  = 21
 *    write(1, "read value: [1] 0x69\n", 21)  = 21
 *    write(1, "read value: [2] 0xa\n", 20)   = 20
 *    poll([{fd=0, events=POLLIN}], 1, 60000) = ? ERESTART_RESTARTBLOCK (To be restarted)
 *    --- SIGUSR1 (User defined signal 1) @ 0 (0) ---
 *    write(2, "Handling signal 10\n", 19)    = 19
 *    sigreturn()                             = ? (mask now [])
 *    write(2, "error in poll: 4 (Interrupted sy"..., 43) = 43
 *    poll([{fd=0, events=POLLIN}], 1, 60000)
 *
 */

#include <signal.h>
#include <stdio.h>
#include <poll.h>
#include <unistd.h>
#include <assert.h>
#include <errno.h>
#include <string.h>


void sighandler(int sig) {
        fprintf(stderr, "Handling signal %d\n", sig);
        fflush(stderr);
}

int main(int argc, char** argv)
{
	struct pollfd fds[1];
	struct sigaction sa;
	sa.sa_handler = sighandler;
	sa.sa_flags = SA_RESTART; /* SA_RESTART ignored by poll */
	assert(sigaction(SIGUSR1, &sa, NULL) == 0);

	while (1) {
		int ret;

		fds[0].fd = 0;  /* stdin */
		fds[0].events = POLLIN;
		fds[0].revents = 0;

		ret = poll(fds, 1, 60 * 1000);
		if (ret == 0) {
			fprintf(stderr, "timeout\n");
		} else if (ret >= 1) {
			if (fds[0].revents == POLLIN) {
				/* Data for reading available */
				char buf[10];
				int len;
				len = read(0, &buf, sizeof(buf));
				if (len >= 1) {
					for(int i = 0; i < len; i++)
						printf("read value: [%d] 0x%hhx\n", i, buf[i]);
				} else if (len == 0) {
					fprintf(stderr, "End of file reached\n");
					break;
				} else {
					fprintf(stderr, "Error in read: %d\n", len);
				}
			} else {
				fprintf(stderr, "Unhandled revents value!\n");
			}
		} else {
			fprintf(stderr, "error in poll: %d (%s)\n", errno, strerror(errno));
		}
	}

	return 0;
}
